from keras import activations, layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion


@register_keras_serializable(package='TFSwin')
class MLP(layers.Layer):
    def __init__(self, ratio, dropout, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=3)

        self.ratio = ratio
        self.dropout = dropout

    @shape_type_conversion
    def build(self, input_shape):
        channels = input_shape[-1]
        if channels is None:
            raise ValueError('Channel dimension of the inputs should be defined. Found `None`.')
        self.input_spec = layers.InputSpec(ndim=3, axes={-1: channels})

        # noinspection PyAttributeOutsideInit
        self.fc1 = layers.Dense(int(channels * self.ratio), name='fc1')

        # noinspection PyAttributeOutsideInit
        self.fc2 = layers.Dense(channels, name='fc2')

        # noinspection PyAttributeOutsideInit
        self.drop = layers.Dropout(self.dropout)

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        outputs = self.fc1(inputs)
        outputs = activations.gelu(outputs)
        outputs = self.drop(outputs)
        outputs = self.fc2(outputs)
        outputs = self.drop(outputs)

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        return input_shape

    def get_config(self):
        config = super().get_config()
        config.update({
            'ratio': self.ratio,
            'dropout': self.dropout
        })

        return config



from tfswin.model import SwinTransformer
from tfswin.model import SwinTransformerTiny224
from tfswin.model import SwinTransformerSmall224
from tfswin.model import SwinTransformerBase224, SwinTransformerBase384
from tfswin.model import SwinTransformerLarge224, SwinTransformerLarge384
from tfswin.prep import preprocess_input



import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.merge import PatchMerging


@keras_parameterized.run_all_keras_modes
class TestPatchMerging(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            PatchMerging,
            kwargs={},
            input_shape=[2, 12 * 12, 4],
            input_dtype='float32',
            expected_output_shape=[None, 6 * 6, 8],
            expected_output_dtype='float32'
        )


if __name__ == '__main__':
    tf.test.main()



import tensorflow as tf
from keras import layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.norm import LayerNorm


@register_keras_serializable(package='TFSwin')
class PatchEmbedding(layers.Layer):
    def __init__(self, patch_size, embed_dim, normalize, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=4)

        self.patch_size = patch_size
        self.embed_dim = embed_dim
        self.normalize = normalize

        self.num_patches = None

    @shape_type_conversion
    def build(self, input_shape):
        height, width, channels = input_shape[1:]
        if None in {height, width, channels}:
            raise ValueError('Height, width and channel dimensions of the inputs should be defined. Found `None`.')
        if height % self.patch_size or width % self.patch_size:
            raise ValueError('Height and width should be evenly dividable by corresponding patch size.')
        self.input_spec = layers.InputSpec(ndim=4, axes={1: height, 2: width, 3: channels})

        self.num_patches = (height // self.patch_size) * (width // self.patch_size)

        # noinspection PyAttributeOutsideInit
        self.proj = layers.Conv2D(self.embed_dim, kernel_size=self.patch_size, strides=self.patch_size, name='proj')

        if self.normalize:
            # noinspection PyAttributeOutsideInit
            self.norm = LayerNorm(name='norm')

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        outputs = self.proj(inputs)
        outputs = tf.reshape(outputs, [-1, self.num_patches, self.embed_dim])

        if self.normalize:
            outputs = self.norm(outputs)

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        return [input_shape[0], self.num_patches, self.embed_dim]

    def get_config(self):
        config = super().get_config()
        config.update({
            'patch_size': self.patch_size,
            'embed_dim': self.embed_dim,
            'normalize': self.normalize,
        })

        return config



from keras import initializers, layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion


@register_keras_serializable(package='TFSwin')
class AbsoluteEmbedding(layers.Layer):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=3)

    @shape_type_conversion
    def build(self, input_shape):
        num_patches, embed_dim = input_shape[-2:]
        if None in {num_patches, embed_dim}:
            raise ValueError('Number of patches and embedding dimension of the inputs should be defined. Found `None`.')
        self.input_spec = layers.InputSpec(ndim=3, axes={1: num_patches, 2: embed_dim})

        # noinspection PyAttributeOutsideInit
        self.embedding = self.add_weight(
            'embedding',
            shape=[1, num_patches, embed_dim],
            initializer=initializers.TruncatedNormal(stddev=0.02),
            trainable=True,
            dtype=self.dtype)

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        return inputs + self.embedding

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        return input_shape



import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.swin import SwinBlock


@keras_parameterized.run_all_keras_modes
class TestSwinBlock(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            SwinBlock,
            kwargs={'num_heads': 24, 'window_size': 7, 'shift_size': 3, 'mlp_ratio': 4., 'qkv_bias': True,
                    'qk_scale': None, 'drop': 0., 'attn_drop': 0., 'path_drop': 0.20000000298023224},
            input_shape=[2, 49, 768],
            input_dtype='float32',
            expected_output_shape=[None, 49, 768],
            expected_output_dtype='float32'
        )
        testing_utils.layer_test(
            SwinBlock,
            kwargs={'num_heads': 24, 'window_size': 7, 'shift_size': 8, 'mlp_ratio': 4., 'qkv_bias': True,
                    'qk_scale': None, 'drop': 0., 'attn_drop': 0., 'path_drop': 0.20000000298023224},
            input_shape=[2, 49, 768],
            input_dtype='float32',
            expected_output_shape=[None, 49, 768],
            expected_output_dtype='float32'
        )


if __name__ == '__main__':
    tf.test.main()



import numpy as np
import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.winatt import WindowAttention


@keras_parameterized.run_all_keras_modes
class TestWindowAttention(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            WindowAttention,
            kwargs={'window_size': 7, 'num_heads': 3, 'qkv_bias': True, 'qk_scale': None, 'attn_mask': False,
                    'attn_drop': 0., 'proj_drop': 0.},
            input_shape=[2, 49, 96],
            input_dtype='float32',
            expected_output_shape=[None, 49, 96],
            expected_output_dtype='float32'
        )

    def test_masked(self):
        kwargs = {'window_size': 7, 'num_heads': 24, 'qkv_bias': True, 'qk_scale': None, 'attn_mask': True,
                  'attn_drop': 0., 'proj_drop': 0.}
        layer = WindowAttention(**kwargs)
        inputs = [np.zeros((2, 49, 96), 'float32'), np.zeros((1, 49, 49), 'float32')]

        outputs = layer(inputs)
        outputs = self.evaluate(outputs)
        self.assertTupleEqual(outputs.shape, (2, 49, 96))
        self.assertEqual(outputs.dtype, inputs[0].dtype)


if __name__ == '__main__':
    tf.test.main()



import numpy as np
import tensorflow as tf
from keras import layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.norm import LayerNorm


@register_keras_serializable(package='TFSwin')
class PatchMerging(layers.Layer):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=3)

        self.length = None
        self.channels = None
        self.size = None

    @shape_type_conversion
    def build(self, input_shape):
        self.length, self.channels = input_shape[1:]
        if None in {self.length, self.channels}:
            raise ValueError('Length and channel dimensions of the inputs should be defined. Found `None`.')
        self.input_spec = layers.InputSpec(ndim=3, axes={1: self.length, 2: self.channels})

        self.size = int(self.length ** 0.5)
        if self.size ** 2 != self.length:
            raise ValueError('Height and width of the inputs should be equal.')
        if self.size % 2:
            raise ValueError('Height and width should be evenly dividable by 2.')

        # noinspection PyAttributeOutsideInit
        self.norm = LayerNorm(name='norm')

        # noinspection PyAttributeOutsideInit
        self.reduction = layers.Dense(self.channels * 2, use_bias=False, name='reduction')

        indices = np.arange(0, self.length).reshape((self.size, self.size))
        # noinspection PyAttributeOutsideInit
        self.indices = np.stack([
            indices[0::2, 0::2],  # B H/2 W/2 C
            indices[1::2, 0::2],  # B H/2 W/2 C
            indices[0::2, 1::2],  # B H/2 W/2 C
            indices[1::2, 1::2]  # B H/2 W/2 C
        ], axis=-1).ravel()

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        outputs = tf.gather(inputs, self.indices, axis=1)
        outputs = tf.reshape(outputs, [-1, self.length // 4, self.channels * 4])

        outputs = self.norm(outputs)
        outputs = self.reduction(outputs)

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        out_length = None if self.length is None else self.length // 4
        out_channels = None if self.channels is None else self.channels * 2

        return [input_shape[0], out_length, out_channels]



import numpy as np
from keras import layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.merge import PatchMerging
from tfswin.swin import SwinBlock


@register_keras_serializable(package='TFSwin')
class BasicLayer(layers.Layer):
    def __init__(self, depth, num_heads, window_size, mlp_ratio=4., qkv_bias=True, qk_scale=None, drop=0., attn_drop=0.,
                 path_drop=0., downsample=False, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=3)

        self.depth = depth
        self.num_heads = num_heads
        self.window_size = window_size
        self.mlp_ratio = mlp_ratio
        self.qkv_bias = qkv_bias
        self.qk_scale = qk_scale
        self.drop = drop
        self.attn_drop = attn_drop
        self.path_drop = path_drop
        self.downsample = downsample

    @shape_type_conversion
    def build(self, input_shape):
        path_drop = self.path_drop
        if not isinstance(self.path_drop, (list, tuple)):
            path_drop = [self.path_drop] * self.depth

        shift_size = np.zeros(self.depth, 'int32')
        shift_size[1::2] = self.window_size // 2

        # noinspection PyAttributeOutsideInit
        self.blocks = [
            SwinBlock(num_heads=self.num_heads, window_size=self.window_size, shift_size=shift_size[i],
                      mlp_ratio=self.mlp_ratio, qkv_bias=self.qkv_bias, qk_scale=self.qk_scale,
                      drop=self.drop, attn_drop=self.attn_drop, path_drop=path_drop[i], name=f'blocks.{i}')
            for i in range(self.depth)]

        if self.downsample:
            # noinspection PyAttributeOutsideInit
            self.down = PatchMerging(name='downsample')

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        outputs = inputs
        for b in self.blocks:
            outputs = b(outputs)

        if self.downsample:
            outputs = self.down(outputs)

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        if self.downsample:
            return self.down.compute_output_shape(input_shape)

        return input_shape

    def get_config(self):
        config = super().get_config()
        config.update({
            'depth': self.depth,
            'num_heads': self.num_heads,
            'window_size': self.window_size,
            'mlp_ratio': self.mlp_ratio,
            'qkv_bias': self.qkv_bias,
            'qk_scale': self.qk_scale,
            'drop': self.drop,
            'attn_drop': self.attn_drop,
            'path_drop': self.path_drop,
            'downsample': self.downsample,
        })

        return config



import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.basic import BasicLayer


@keras_parameterized.run_all_keras_modes
class TestBasicLayer(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            BasicLayer,
            kwargs={'depth': 2, 'num_heads': 3, 'window_size': 7, 'mlp_ratio': 4., 'qkv_bias': True, 'qk_scale': None,
                    'drop': 0., 'attn_drop': 0., 'path_drop': [0.0, 0.0181818176060915], 'downsample': True},
            input_shape=[2, 3136, 96],
            input_dtype='float32',
            expected_output_shape=[None, 784, 192],
            expected_output_dtype='float32'
        )
        testing_utils.layer_test(
            BasicLayer,
            kwargs={'depth': 2, 'num_heads': 3, 'window_size': 7, 'mlp_ratio': 4., 'qkv_bias': True, 'qk_scale': None,
                    'drop': 0., 'attn_drop': 0., 'path_drop': [0.0, 0.0181818176060915], 'downsample': False},
            input_shape=[2, 3136, 96],
            input_dtype='float32',
            expected_output_shape=[None, 3136, 96],
            expected_output_dtype='float32'
        )


if __name__ == '__main__':
    tf.test.main()



import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.norm import LayerNorm


@keras_parameterized.run_all_keras_modes
class TestLayerNorm(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            LayerNorm,
            kwargs={},
            input_shape=[2, 16, 16, 3],
            input_dtype='float32',
            expected_output_shape=[None, 16, 16, 3],
            expected_output_dtype='float32'
        )
        testing_utils.layer_test(
            LayerNorm,
            kwargs={},
            input_shape=[2, 16, 16, 3],
            input_dtype='float16',
            expected_output_shape=[None, 16, 16, 3],
            expected_output_dtype='float16'
        )


if __name__ == '__main__':
    tf.test.main()



import numpy as np
import tensorflow as tf
from keras import layers
from keras.utils.generic_utils import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.drop import DropPath
from tfswin.mlp import MLP
from tfswin.norm import LayerNorm
from tfswin.winatt import WindowAttention


@register_keras_serializable(package='TFSwin')
class SwinBlock(layers.Layer):
    def __init__(self, num_heads, window_size=7, shift_size=0, mlp_ratio=4., qkv_bias=True, qk_scale=None, drop=0.,
                 attn_drop=0., path_drop=0., **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=3)

        self.num_heads = num_heads
        self.window_size = window_size
        self.shift_size = shift_size
        self.mlp_ratio = mlp_ratio
        self.qkv_bias = qkv_bias
        self.qk_scale = qk_scale
        self.drop = drop
        self.attn_drop = attn_drop
        self.path_drop = path_drop

    @shape_type_conversion
    def build(self, input_shape):
        # noinspection PyAttributeOutsideInit
        self.length, self.channels = input_shape[1:]
        if None in {self.length, self.channels}:
            raise ValueError('Length and channel dimensions of the inputs should be defined. Found `None`.')
        self.input_spec = layers.InputSpec(ndim=3, axes={1: self.length, 2: self.channels})

        # noinspection PyAttributeOutsideInit
        self.size = int(self.length ** 0.5)
        if self.size ** 2 != self.length:
            raise ValueError('Height and width of the inputs should be equal.')

        self._shift_size = self.shift_size
        if self.size <= self.window_size:
            # noinspection PyAttributeOutsideInit
            self._shift_size = 0
            self.window_size = self.size
        if not 0 <= self._shift_size < self.window_size:
            raise ValueError('Shift size must be in range [0; window_size).')

        # noinspection PyAttributeOutsideInit
        self.norm1 = LayerNorm(name='norm1')

        # noinspection PyAttributeOutsideInit
        self.attn = WindowAttention(window_size=self.window_size, num_heads=self.num_heads, qkv_bias=self.qkv_bias,
                                    qk_scale=self.qk_scale, attn_mask=bool(self._shift_size), attn_drop=self.attn_drop,
                                    proj_drop=self.drop, name='attn')

        # noinspection PyAttributeOutsideInit
        self.drop_path = DropPath(self.path_drop)

        # noinspection PyAttributeOutsideInit
        self.norm2 = LayerNorm(name='norm2')

        # noinspection PyAttributeOutsideInit
        self.mlp = MLP(ratio=self.mlp_ratio, dropout=self.drop, name='mlp')

        super().build(input_shape)

    def attn_mask(self):
        img_mask = np.zeros([1, self.size, self.size, 1], 'float32')
        h_slices = (slice(0, -self.window_size), slice(-self.window_size, -self._shift_size),
                    slice(-self._shift_size, None))
        w_slices = (slice(0, -self.window_size), slice(-self.window_size, -self._shift_size),
                    slice(-self._shift_size, None))
        cnt = 0
        for h in h_slices:
            for w in w_slices:
                img_mask[:, h, w, :] = cnt
                cnt += 1

        mask_windows = window_partition(img_mask, self.window_size, 'float32')[..., 0]
        attn_mask = mask_windows[:, None] - mask_windows[:, :, None]
        attn_mask = tf.where(attn_mask == 0., 0., -100.)
        attn_mask = tf.cast(attn_mask, self.compute_dtype)

        return attn_mask

    def call(self, inputs, *args, **kwargs):
        outputs = self.norm1(inputs)
        outputs = tf.reshape(outputs, [-1, self.size, self.size, self.channels])

        # Cyclic shift
        if self._shift_size > 0:
            outputs = tf.roll(outputs, [-self._shift_size, -self._shift_size], [1, 2])

        # Partition windows
        outputs = window_partition(outputs, self.window_size, self.compute_dtype)

        # W-MSA/SW-MSA
        if self._shift_size:
            outputs = self.attn([outputs, self.attn_mask()])
        else:
            outputs = self.attn(outputs)

        # Merge windows
        outputs = window_reverse(outputs, self.size, self.compute_dtype)

        # Reverse cyclic shift
        if self._shift_size > 0:
            outputs = tf.roll(outputs, [self._shift_size, self._shift_size], [1, 2])

        # FFN
        outputs = tf.reshape(outputs, [-1, self.length, self.channels])
        outputs = inputs + self.drop_path(outputs)
        outputs += self.drop_path(self.mlp(self.norm2(outputs)))

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        return input_shape

    def get_config(self):
        config = super().get_config()
        config.update({
            'num_heads': self.num_heads,
            'window_size': self.window_size,
            'shift_size': self.shift_size,
            'mlp_ratio': self.mlp_ratio,
            'qkv_bias': self.qkv_bias,
            'qk_scale': self.qk_scale,
            'drop': self.drop,
            'attn_drop': self.attn_drop,
            'path_drop': self.path_drop,
        })

        return config


def window_partition(inputs, window_size, dtype=None, name=None):
    with tf.name_scope(name or 'window_partition'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 4 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 4.')

        if None in set(inputs.shape[1:]):
            raise ValueError('Height, width and channel dimensions of the inputs should be defined. Found `None`.')

        height, width, channels = inputs.shape[1:]

        if height != width:
            raise ValueError('Height and width of the inputs should be equal.')
        num_windows = height // window_size

        outputs = tf.reshape(inputs, [-1, num_windows, window_size, num_windows, window_size, channels])
        outputs = tf.transpose(outputs, [0, 1, 3, 2, 4, 5])
        outputs = tf.reshape(outputs, [-1, window_size ** 2, channels])

        return outputs


def window_reverse(inputs, size, dtype=None, name=None):
    with tf.name_scope(name or 'window_reverse'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 3 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 3.')

        if None in set(inputs.shape[1:]):
            raise ValueError('Length and channel dimensions of the inputs should be defined. Found `None`.')

        length, channels = inputs.shape[1:]

        window_size = int(length ** 0.5)
        if window_size ** 2 != length:
            raise ValueError('Length should be equal to window size ^ 2.')

        num_windows = size // window_size

        outputs = tf.reshape(inputs, [-1, num_windows, num_windows, window_size, window_size, channels])
        outputs = tf.transpose(outputs, [0, 1, 3, 2, 4, 5])
        outputs = tf.reshape(outputs, [-1, size, size, channels])

        return outputs



import tensorflow as tf
import warnings
from keras import layers
from keras.utils.generic_utils import register_keras_serializable


@register_keras_serializable(package='TFSwin')
class LayerNorm(layers.LayerNormalization):
    # Overload defaults and casting to use fused implementation

    def __init__(self, epsilon=1.001e-5, dtype='float32', **kwargs):
        kwargs['autocast'] = False
        super().__init__(epsilon=epsilon, dtype=dtype, **kwargs)

    def build(self, input_shape):
        super().build(input_shape)
        if not self._fused:
            warnings.warn(f'Layer {self.name} will use an inefficient implementation.')

    def call(self, inputs, *args, **kwargs):
        outputs = tf.cast(inputs, 'float32')

        outputs = super().call(outputs)

        if inputs.dtype == tf.dtypes.float16:
            outputs = tf.clip_by_value(outputs, tf.dtypes.float16.min, tf.dtypes.float16.max)
        outputs = tf.cast(outputs, inputs.dtype)

        return outputs

    def compute_output_signature(self, input_signature):
        return input_signature



import numpy as np
import tensorflow as tf
from keras import backend, layers, models
from keras.applications import imagenet_utils
from keras.utils import data_utils, layer_utils
from tfswin.ape import AbsoluteEmbedding
from tfswin.basic import BasicLayer
from tfswin.embed import PatchEmbedding
from tfswin.norm import LayerNorm

BASE_URL = 'https://github.com/shkarupa-alex/tfswin/releases/download/1.0.0/swin_{}.h5'
WEIGHT_URLS = {
    'swin_tiny_224': BASE_URL.format('tiny_patch4_window7_224'),
    'swin_small_224': BASE_URL.format('small_patch4_window7_224'),
    'swin_base_224': BASE_URL.format('base_patch4_window7_224_22k'),
    'swin_base_384': BASE_URL.format('base_patch4_window12_384_22k'),
    'swin_large_224': BASE_URL.format('large_patch4_window7_224_22k'),
    'swin_large_384': BASE_URL.format('large_patch4_window12_384_22k')
}
WEIGHT_HASHES = {
    'swin_tiny_224': 'd4978388fed2c48d3e141ff8bfadfd402626d6e405d5a89aa146cd2dd6bef04f',
    'swin_small_224': 'd4dc852d32c00762f0ba5c6b33d3a08670c3aa3377c5d10104d505260070967e',
    'swin_base_224': 'b68200eb707302e471242da86fd5a9d3b84d0c9e30698bd193b775207b5100eb',
    'swin_base_384': '48dd1a39217ac26a7c88ebe1212714812f15db6ca97ec09bed0ac85cbdf930f1',
    'swin_large_224': '8910d7390d275a06a4c1a65ab458b089f72c431351ccfec94c296f18cbcdd714',
    'swin_large_384': 'a04eb046fd5412a1f4339478e625b8f691d9811340a98b7675295fb7d1480c77'
}


def SwinTransformer(
        model_name, input_shape, window_size, embed_dim, depths, num_heads, patch_size=4, patch_norm=True,
        use_ape=False, drop_rate=0., mlp_ratio=4., qkv_bias=True, qk_scale=None, attn_drop=0., path_drop=0.1,
        include_top=True, classes=1000, classifier_activation='softmax', pooling=None, weights=None, input_tensor=None):
    """Instantiates the Swin Transformer architecture.

    Args:
      model_name: model name.
      input_shape: fully-defined shape tuple of an input image. Height and width should be equal.
      window_size: window partition size.
      embed_dim: patch embedding dimension.
      depths: depth of each Swin Transformer layer.
      num_heads: number of attention heads.
      patch_size: patch size used to divide input image.
      patch_norm: whether to add normalization after patch embedding.
      use_ape: whether to add absolute position embedding to the patch embedding
      drop_rate: dropout rate.
      mlp_ratio: ratio of mlp hidden units to embedding units.
      qkv_bias: whether to add a learnable bias to query, key, value.
      qk_scale: override default qk scale of head_dim ** -0.5 if set
      attn_drop: attention dropout rate
      path_drop: stochastic depth rate
      include_top: whether to include the fully-connected layer at the top of the network.
      classes: optional number of classes to classify images into, only to be specified if `include_top` is True.
      classifier_activation: the activation function to use on the "top" layer. Ignored unless `include_top=True`.
        When loading pretrained weights, `classifier_activation` can only be `None` or `"softmax"`.
      pooling: optional pooling mode for feature extraction when `include_top` is `False`.
        - `None` means that the output of the model will be the 3D tensor output of the last layer.
        - `avg` means that global average pooling will be applied to the output of the last layer, and thus the output
          of the model will be a 2D tensor.
        - `max` means that global max pooling will be applied.
      weights: one of `None` (random initialization), 'imagenet' (pre-training on ImageNet or ImageNet 21k), or the
        path to the weights file to be loaded.
      input_tensor: optional Keras tensor (i.e. output of `layers.Input()`) to use as image input for the model.

    Returns:
      A `keras.Model` instance.
    """
    if not (weights in {'imagenet', None} or tf.io.gfile.exists(weights)):
        raise ValueError('The `weights` argument should be either `None` (random initialization), `imagenet` '
                         '(pre-training on ImageNet), or the path to the weights file to be loaded.')

    if weights == 'imagenet' and include_top and classes not in {1000, 21841}:
        raise ValueError('If using `weights` as `"imagenet"` with `include_top` as true, '
                         '`classes` should be 1000 or 21841 depending on model type')

    if input_tensor is not None:
        try:
            backend.is_keras_tensor(input_tensor)
        except ValueError:
            raise ValueError(f'Expecting `input_tensor` to be a symbolic tensor instance. '
                             f'Got {input_tensor} of type {type(input_tensor)}')

    if input_tensor is not None:
        tensor_shape = backend.int_shape(input_tensor)[1:]
        if input_tensor.dtype != 'float32':
            raise ValueError('DType of `input_tensor` should equals to `float32`.')

        if input_shape and tensor_shape != input_shape:
            raise ValueError('Shape of `input_tensor` should equals to `input_shape` if both provided.')
        else:
            input_shape = tensor_shape

    if input_shape[0] != input_shape[1]:
        raise ValueError('Height and width of the inputs should be equal.')

    # Determine proper input shape
    input_shape = imagenet_utils.obtain_input_shape(
        input_shape,
        default_size=224,
        min_size=32,
        data_format='channel_last',
        require_flatten=False,
        weights=weights)

    if input_tensor is not None:
        if backend.is_keras_tensor(input_tensor):
            image = input_tensor
        else:
            image = layers.Input(tensor=input_tensor, shape=input_shape, dtype='float32')
    else:
        image = layers.Input(shape=input_shape)

    # Define model pipeline
    x = PatchEmbedding(patch_size=patch_size, embed_dim=embed_dim, normalize=patch_norm, name='patch_embed')(image)

    if use_ape:
        x = AbsoluteEmbedding()(x)

    x = layers.Dropout(drop_rate, name='pos_drop')(x)

    path_drops = [x for x in np.linspace(0., path_drop, sum(depths))]

    for i in range(len(depths)):
        path_drop = path_drops[sum(depths[:i]):sum(depths[:i + 1])]
        not_last = i != len(depths) - 1
        x = BasicLayer(depth=depths[i], num_heads=num_heads[i], window_size=window_size, mlp_ratio=mlp_ratio,
                       qkv_bias=qkv_bias, qk_scale=qk_scale, drop=drop_rate, attn_drop=attn_drop,
                       path_drop=path_drop, downsample=not_last, name=f'layers.{i}')(x)

    x = LayerNorm(name='norm')(x)

    if include_top or pooling in {None, 'avg'}:
        x = layers.GlobalAveragePooling1D(name='avg_pool')(x)
    elif pooling == 'max':
        x = layers.GlobalMaxPooling1D(name='max_pool')(x)
    else:
        raise ValueError(f'Expecting pooling to be one of None/avg/max. Found: {pooling}')

    imagenet_utils.validate_activation(classifier_activation, weights)
    x = layers.Dense(classes, name='head')(x)
    x = layers.Activation(classifier_activation, dtype='float32', name='pred')(x)

    # Ensure that the model takes into account any potential predecessors of `input_tensor`.
    if input_tensor is not None:
        inputs = layer_utils.get_source_inputs(input_tensor)
    else:
        inputs = image

    # Create model.
    model = models.Model(inputs, x, name=model_name)

    # Load weights.
    if 'imagenet' == weights and model_name in WEIGHT_URLS:
        weights_url = WEIGHT_URLS[model_name]
        weights_hash = WEIGHT_HASHES[model_name]
        weights_path = data_utils.get_file(origin=weights_url, file_hash=weights_hash, cache_subdir='tfswin')
        model.load_weights(weights_path)
    elif weights is not None:
        model.load_weights(weights)

    if include_top:
        return model

    last_layer = 'norm'
    if pooling == 'avg':
        last_layer = 'avg_pool'
    elif pooling == 'max':
        last_layer = 'max_pool'

    outputs = model.get_layer(name=last_layer).output
    model = models.Model(inputs=inputs, outputs=outputs, name=model_name)

    return model


def SwinTransformerTiny224(model_name='swin_tiny_224', input_shape=(224, 224, 3), window_size=7, embed_dim=96,
                           depths=(2, 2, 6, 2), num_heads=(3, 6, 12, 24), path_drop=0.2, weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, path_drop=path_drop, weights=weights, **kwargs)


def SwinTransformerSmall224(model_name='swin_small_224', input_shape=(224, 224, 3), window_size=7, embed_dim=96,
                            depths=(2, 2, 18, 2), num_heads=(3, 6, 12, 24), path_drop=0.3, weights='imagenet',
                            **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, path_drop=path_drop, weights=weights, **kwargs)


def SwinTransformerBase224(model_name='swin_base_224', input_shape=(224, 224, 3), window_size=7, embed_dim=128,
                           depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), path_drop=0.5, classes=21841,
                           weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, path_drop=path_drop, weights=weights, classes=classes,
                           **kwargs)


def SwinTransformerBase384(model_name='swin_base_384', input_shape=(384, 384, 3), window_size=12, embed_dim=128,
                           depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), classes=21841, weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, weights=weights, classes=classes, **kwargs)


def SwinTransformerLarge224(model_name='swin_large_224', input_shape=(224, 224, 3), window_size=7, embed_dim=192,
                            depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), classes=21841, weights='imagenet',
                            **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, weights=weights, classes=classes, **kwargs)


def SwinTransformerLarge384(model_name='swin_large_384', input_shape=(384, 384, 3), window_size=12, embed_dim=192,
                            depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), classes=21841, weights='imagenet',
                            **kwargs):
    return SwinTransformer(model_name=model_name, input_shape=input_shape, window_size=window_size, embed_dim=embed_dim,
                           depths=depths, num_heads=num_heads, weights=weights, classes=classes, **kwargs)



import tensorflow as tf
from keras import keras_parameterized, testing_utils
from tfswin.embed import PatchEmbedding


@keras_parameterized.run_all_keras_modes
class TestPatchEmbedding(keras_parameterized.TestCase):
    def test_layer(self):
        testing_utils.layer_test(
            PatchEmbedding,
            kwargs={'patch_size': 4, 'embed_dim': 2, 'normalize': False},
            input_shape=[2, 12, 12, 3],
            input_dtype='float32',
            expected_output_shape=[None, 9, 2],
            expected_output_dtype='float32'
        )
        testing_utils.layer_test(
            PatchEmbedding,
            kwargs={'patch_size': 3, 'embed_dim': 2, 'normalize': True},
            input_shape=[2, 12, 12, 3],
            input_dtype='float32',
            expected_output_shape=[None, 16, 2],
            expected_output_dtype='float32'
        )


if __name__ == '__main__':
    tf.test.main()
